VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsIPropertyStore"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 09/2018

' IUnknown interface
' https://docs.microsoft.com/en-us/windows/desktop/api/unknwn/nn-unknwn-iunknown

' IPropertyStore interface
' https://msdn.microsoft.com/de-de/library/windows/desktop/bb761474(v=vs.85).aspx

Option Explicit

' ----==== Const ====----
Private Const ERR_BASE As Long = 40670
Private Const CLSID_PropertyStore As String = ""
Private Const IID_IPropertyStore As String = "{886D8EEB-8CF2-4446-8D02-CDBA1DBDCF99}"

Private Enum E_vtbOffsets
    '/*** IUnknown methods ***/
    vtb_QueryInterface
    vtb_AddRef
    vtb_Release
    '/*** IPropertyStore methods ***/
    vtb_GetCount
    vtb_GetAt
    vtb_GetValue
    vtb_SetValue
    vtb_Commit
End Enum

' ----==== Variablen ====----
Private m_initialized As Boolean
Private m_Interface As Interface_Data

Public Function GetCount() As Long

    Dim lRet As Long
    Dim cProps As Long
    Dim pcProps As Long

    pcProps = VarPtr(cProps)

    If x_OleInvoke(vtb_GetCount, pcProps) Then

        lRet = cProps

    End If

    GetCount = lRet

End Function

Friend Function GetAt(ByVal iProp As Long) As PROPERTYKEY

    Dim tRet As PROPERTYKEY
    Dim tPROPERTYKEY As PROPERTYKEY
    Dim ptPROPERTYKEY As Long

    ptPROPERTYKEY = VarPtr(tPROPERTYKEY)

    If x_OleInvoke(vtb_GetAt, iProp, ptPROPERTYKEY) Then

        tRet = tPROPERTYKEY

    End If

    GetAt = tRet

End Function

Friend Function GetValue(ByRef key As PROPERTYKEY) As PROPVARIANT

    Dim tRet As PROPVARIANT
    Dim pkey As Long
    Dim tPROPVARIANT As PROPVARIANT
    Dim ptPROPVARIANT As Long

    pkey = VarPtr(key)
    ptPROPVARIANT = VarPtr(tPROPVARIANT)

    If x_OleInvoke(vtb_GetValue, pkey, ptPROPVARIANT) Then

        tRet = tPROPVARIANT

    End If

    GetValue = tRet

End Function

Friend Function SetValue(ByRef key As PROPERTYKEY, ByRef propvar As PROPVARIANT) As Boolean
    
    Dim bRet As Boolean
    Dim pkey As Long
    Dim ppropvar As Long
    
    pkey = VarPtr(key)
    ppropvar = VarPtr(propvar)
    
    If x_OleInvoke(vtb_SetValue, pkey, ppropvar) Then

        bRet = True

    End If

    SetValue = bRet

End Function

Public Function Commit() As Boolean

    Dim bRet As Boolean
    
    If x_OleInvoke(vtb_Commit) Then

        bRet = True

    End If

    Commit = bRet

End Function

' ----==== Helper Func ====----
Public Sub Initialize(ByVal pInterface As Long)

    m_Interface.owner = ObjPtr(Me)
    m_Interface.RaiseErrors = False
    m_Interface.ifc = pInterface
    m_initialized = True

End Sub

Private Sub Class_Terminate()

    Call ReleaseInterface(m_Interface)

End Sub

Public Sub RaiseError(Optional ByVal Raise As Boolean = False)

    m_Interface.RaiseErrors = Raise

End Sub

Private Function x_OleInvoke(ByVal vtb As E_vtbOffsets, ParamArray var()) As Boolean

    Dim ret As Long

    If Not m_initialized Then

        Call InterfaceError(m_Interface, ecd_OleInvoke)

    Else

        x_OleInvoke = oleInvoke(m_Interface, vtb, ret, True, var)

    End If

End Function

Public Sub x_RaiseError()

    With m_Interface

        If .ecd Then Err.Raise ERR_BASE + .ecd, App.EXEName & _
            ".clsIPropertyStore", .etx

        .ecd = 0
        .etx = vbNullString

    End With

End Sub

