VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsIShellItem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 11/2017

' IUnknown interface
' https://docs.microsoft.com/en-us/windows/desktop/api/unknwn/nn-unknwn-iunknown

' IShellItem interface
' https://docs.microsoft.com/en-us/windows/desktop/api/shobjidl_core/nn-shobjidl_core-ishellitem

Option Explicit

' ----==== Const ====----
Private Const ERR_BASE As Long = 40670
Private Const CLSID_ShellItem As String = "{9ac9fbe1-e0a2-4ad6-b4ee-e212013ea917}"
Private Const IID_IShellItem As String = "{43826d1e-e718-42ee-bc55-a1e261c37bfe}"

' Die Positionen der einzelnen vtbOffets ergeben sich aus dem entsprechende Header
' zum Interface. Siehe dazu zB. -> typedef struct IShellItemVtbl. In der Funktion
' oleInvoke wird dann der Offset * 4 genommen um dann per DispCallFunc die richtige
' Funktion im Interface aufzurufen.
' Interface Pointer + (Offset * 4) = Pointer zur Funktion
Private Enum E_vtbOffsets
    '/*** IUnknown methods ***/     ' Offset
    vtb_QueryInterface              ' 0
    vtb_AddRef                      ' 1
    vtb_Release                     ' 2
    '/*** IShellItem methods ***/
    vtb_BindToHandler               ' 3
    vtb_GetParent                   ' 4
    vtb_GetDisplayName              ' 5
    vtb_GetAttributes               ' 6
    vtb_Compare                     ' 7
End Enum

' Beschreibung des IShellItem-Interfaces aus der Headerdatei ShObjIdl_core.h
'
'EXTERN_C const IID IID_IShellItem;
'
'#if defined(__cplusplus) && !defined(CINTERFACE)
'
'    MIDL_INTERFACE ("43826d1e-e718-42ee-bc55-a1e261c37bfe")
'IShellItem:      Public IUnknown
'    {
'    public:
'        virtual HRESULT STDMETHODCALLTYPE BindToHandler(
'            /* [unique][in] */ __RPC__in_opt IBindCtx *pbc,
'            /* [in] */ __RPC__in REFGUID bhid,
'            /* [in] */ __RPC__in REFIID riid,
'            /* [iid_is][out] */ __RPC__deref_out_opt void **ppv) = 0;
'
'        virtual HRESULT STDMETHODCALLTYPE GetParent(
'            /* [out] */ __RPC__deref_out_opt IShellItem **ppsi) = 0;
'
'        virtual HRESULT STDMETHODCALLTYPE GetDisplayName(
'            /* [in] */ SIGDN sigdnName,
'            /* [annotation][string][out] */
'            _Outptr_result_nullonfailure_  LPWSTR *ppszName) = 0;
'
'        virtual HRESULT STDMETHODCALLTYPE GetAttributes(
'            /* [in] */ SFGAOF sfgaoMask,
'            /* [out] */ __RPC__out SFGAOF *psfgaoAttribs) = 0;
'
'        virtual HRESULT STDMETHODCALLTYPE Compare(
'            /* [in] */ __RPC__in_opt IShellItem *psi,
'            /* [in] */ SICHINTF hint,
'            /* [out] */ __RPC__out int *piOrder) = 0;
'
'    };
'
'
'#else   /* C style interface */
'
'    typedef struct IShellItemVtbl
'    {
'        BEGIN_INTERFACE
'
'        HRESULT ( STDMETHODCALLTYPE *QueryInterface )(
'            __RPC__in IShellItem * This,
'            /* [in] */ __RPC__in REFIID riid,
'            /* [annotation][iid_is][out] */
'            _COM_Outptr_  void **ppvObject);
'
'        ULONG ( STDMETHODCALLTYPE *AddRef )(
'            __RPC__in IShellItem * This);
'
'        ULONG ( STDMETHODCALLTYPE *Release )(
'            __RPC__in IShellItem * This);
'
'        HRESULT ( STDMETHODCALLTYPE *BindToHandler )(
'            __RPC__in IShellItem * This,
'            /* [unique][in] */ __RPC__in_opt IBindCtx *pbc,
'            /* [in] */ __RPC__in REFGUID bhid,
'            /* [in] */ __RPC__in REFIID riid,
'            /* [iid_is][out] */ __RPC__deref_out_opt void **ppv);
'
'        HRESULT ( STDMETHODCALLTYPE *GetParent )(
'            __RPC__in IShellItem * This,
'            /* [out] */ __RPC__deref_out_opt IShellItem **ppsi);
'
'        HRESULT ( STDMETHODCALLTYPE *GetDisplayName )(
'            __RPC__in IShellItem * This,
'            /* [in] */ SIGDN sigdnName,
'            /* [annotation][string][out] */
'            _Outptr_result_nullonfailure_  LPWSTR *ppszName);
'
'        HRESULT ( STDMETHODCALLTYPE *GetAttributes )(
'            __RPC__in IShellItem * This,
'            /* [in] */ SFGAOF sfgaoMask,
'            /* [out] */ __RPC__out SFGAOF *psfgaoAttribs);
'
'        HRESULT ( STDMETHODCALLTYPE *Compare )(
'            __RPC__in IShellItem * This,
'            /* [in] */ __RPC__in_opt IShellItem *psi,
'            /* [in] */ SICHINTF hint,
'            /* [out] */ __RPC__out int *piOrder);
'
'        END_INTERFACE
'    } IShellItemVtbl;
'
'    Interface IShellItem
'    {
'        CONST_VTBL struct IShellItemVtbl *lpVtbl;
'    };

' ----==== Variablen ====----
Private m_initialized As Boolean
Private m_Interface As Interface_Data

Public Function BindToHandler(ByVal pbc As Long, ByVal rbhid As Long, ByVal riid _
    As Long) As Long

    Dim lRet As Long
    Dim ppvOut As Long
    Dim pppvOut As Long

    pppvOut = VarPtr(ppvOut)

    If x_OleInvoke(vtb_BindToHandler, pbc, rbhid, riid, pppvOut) Then

        lRet = ppvOut

    End If

    BindToHandler = lRet

End Function

Public Function GetParent() As Long

    Dim lRet As Long
    Dim ppsi As Long
    Dim pppsi As Long

    pppsi = VarPtr(ppsi)

    If x_OleInvoke(vtb_GetParent, pppsi) Then

        lRet = ppsi

    End If

    GetParent = lRet

End Function

Public Function GetDisplayName(ByVal sigdnName As SIGDN) As String

    Dim sRet As String
    Dim pszName As Long
    Dim ppszName As Long

    ppszName = VarPtr(pszName)

    If x_OleInvoke(vtb_GetDisplayName, sigdnName, ppszName) Then

        sRet = GetStringFromPointer(pszName)

    End If

    GetDisplayName = sRet

End Function

Public Function GetAttributes(ByVal sfgaoMask As Long) As Long

    Dim lRet As Long
    Dim psfgaoAttribs As Long
    Dim ppsfgaoAttribs As Long

    ppsfgaoAttribs = VarPtr(psfgaoAttribs)

    If x_OleInvoke(vtb_GetAttributes, sfgaoMask, ppsfgaoAttribs) Then

        lRet = psfgaoAttribs

    End If

    GetAttributes = lRet

End Function

Public Function Compare(ByVal psi As Long, ByVal hint As SICHINTF) As Long
    
    Dim lRet As Long
    Dim piOrder As Long
    Dim ppiOrder As Long
    
    ppiOrder = VarPtr(piOrder)
    
    If x_OleInvoke(vtb_Compare, psi, hint, ppiOrder) Then

        lRet = piOrder

    End If

    Compare = lRet
    
End Function

' ----==== Helper Func ====----
Public Sub Initialize(ByVal pInterface As Long)

    m_Interface.owner = ObjPtr(Me)  ' Objekt-Pointer dieser Klasse
    m_Interface.RaiseErrors = False ' Bei Fehler keine Unterbrechung
    m_Interface.ifc = pInterface    ' Interface-Pointer
    m_initialized = True            ' ist Initialisiert?
    
    ' Falls kein Pointer auf ein Interface bergeben wird, kann das Interface
    ' aus der CLSID und IID erstellt werden. Typischerweise dann in der
    ' Sub Class_Initialize. Siehe Klasse clsITranscodeImage. Wird hier aber
    ' nicht bentigt da der Pointer von einer externen Funktion kommt.
    
End Sub

Private Sub Class_Terminate()

    ' Ruft die Funktion Release des Interfaces auf
    Call ReleaseInterface(m_Interface)

End Sub

' ndert die Einstellung wenn ein Fehler beim Ausfhren einer
' Interface-Funktion auftritt.
Public Sub RaiseError(Optional ByVal Raise As Boolean = False)

    ' True = Fehler beim Aufrufen von Interface-Funktionen
    ' Unterbricht die weitere Codeausfhrung wenn kein On Error
    ' Statement vorhanden ist. Praktisch beim testen.
    
    ' False = kein Unterbrechen bei einem Fehler. Die entsprechenden
    ' Interface-Funktionen liefern dann bei einem Fehler
    ' x_OleInvoke = False zurck.
    m_Interface.RaiseErrors = Raise

End Sub

' Aufrufen der Interface-Funktion
Private Function x_OleInvoke(ByVal vtb As E_vtbOffsets, ParamArray var()) As Boolean

    Dim ret As Long

    ' ist kein Interface-Pointer vorhanden, also nicht initialisiert
    If Not m_initialized Then

        ' Fehler auslsen
        Call InterfaceError(m_Interface, ecd_OleInvoke)

    Else

        ' Interface-Funktion aufrufen. Siehe modInterface -> oleInvoke
        x_OleInvoke = oleInvoke(m_Interface, vtb, ret, True, var)

    End If

End Function

' wird bei einem Fehler aus der modInterface aufgerufen
Public Sub x_RaiseError()

    With m_Interface

        ' Fehler auslsen
        If .ecd Then Err.Raise ERR_BASE + .ecd, App.EXEName & ".clsIShellItem", .etx

        .ecd = 0    ' Fehlernummer lschen
        .etx = ""   ' Fehlertext lschen

    End With

End Sub

