Attribute VB_Name = "modITranscodeImage"
' Autor: F. Schler (frank@activevb.de)
' Datum: 09/2018

Option Explicit

' ----==== Const ====----
Private Const S_OK As Long = &H0
Private Const VT_UI4 As Long = &H13
Private Const GPS_DEFAULT As Long = &H0
Private Const ImageWidth As String = "System.Image.HorizontalSize"
Private Const ImageHeight As String = "System.Image.VerticalSize"
Private Const IID_IPicture As String = "{7BF80980-BF32-101A-8BBB-00AA00300CAB}"
Private Const IID_IShellItem As String = "{43826d1e-e718-42ee-bc55-a1e261c37bfe}"
Private Const IID_IPropertyStore As String = "{886D8EEB-8CF2-4446-8D02-CDBA1DBDCF99}"
    
' ----==== Enums ====----
Public Enum TI_FLAGS
    TI_BITMAP = &H1
    TI_JPEG = &H2
End Enum

Public Enum SICHINTF
    SICHINT_DISPLAY = &H0
    SICHINT_ALLFIELDS = &H80000000
    SICHINT_CANONICAL = &H10000000
    SICHINT_TEST_FILESYSPATH_IF_NOT_EQUAL = &H20000000
End Enum

Public Enum SIGDN
    SIGDN_NORMALDISPLAY = &H0
    SIGDN_PARENTRELATIVEPARSING = &H80018001
    SIGDN_DESKTOPABSOLUTEPARSING = &H80028000
    SIGDN_PARENTRELATIVEEDITING = &H80031001
    SIGDN_DESKTOPABSOLUTEEDITING = &H8004C000
    SIGDN_FILESYSPATH = &H80058000
    SIGDN_URL = &H80068000
    SIGDN_PARENTRELATIVEFORADDRESSBAR = &H8007C001
    SIGDN_PARENTRELATIVE = &H80080001
    SIGDN_PARENTRELATIVEFORUI = &H80094001
End Enum

Public Enum STGTY
    STGTY_STORAGE = &H1
    STGTY_STREAM = &H2
    STGTY_LOCKBYTES = &H3
    STGTY_PROPERTY = &H4
End Enum

Public Enum STGM
    STGM_FAILIFTHERE = &H0
    STGM_DIRECT = &H0
    STGM_READ = &H0
    STGM_WRITE = &H1
    STGM_READWRITE = &H2
    STGM_SHARE_EXCLUSIVE = &H10
    STGM_SHARE_DENY_WRITE = &H20
    STGM_SHARE_DENY_READ = &H30
    STGM_SHARE_DENY_NONE = &H40
    STGM_CREATE = &H1000
    STGM_TRANSACTED = &H10000
    STGM_CONVERT = &H20000
    STGM_PRIORITY = &H40000
    STGM_NOSCRATCH = &H100000
    STGM_NOSNAPSHOT = &H200000
    STGM_DIRECT_SWMR = &H400000
    STGM_SIMPLE = &H8000000
    STGM_DELETEONRELEASE = &H4000000
End Enum

Public Enum STGC
    STGC_DEFAULT = &H0
    STGC_OVERWRITE = &H1
    STGC_ONLYIFCURRENT = &H2
    STGC_DANGEROUSLYCOMMITMERELYTODISKCACHE = &H4
    STGC_CONSOLIDATE = &H8
End Enum

Public Enum STATFLAG
    STATFLAG_DEFAULT = &H0
    STATFLAG_NONAME = &H1
    STATFLAG_NOOPEN = &H2
End Enum

Public Enum STREAM_SEEK
    STREAM_SEEK_SET = &H0
    STREAM_SEEK_CUR = &H1
    STREAM_SEEK_END = &H2
End Enum

Public Enum LOCKTYPE
    LOCK_WRITE = &H1
    LOCK_EXCLUSIVE = &H2
    LOCK_ONLYONCE = &H4
End Enum

' ----==== Types ====----
Private Type SIZE
    cx As Long
    cy As Long
End Type

Private Type IID
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(0 To 7) As Byte
End Type

Private Type GUID
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(0 To 7) As Byte
End Type

Public Type PROPERTYKEY
    fmtid As GUID
    pid As Long
End Type

Public Type PROPVARIANT
    vt As Integer
    wReserved1 As Integer
    wReserved2 As Integer
    wReserved3 As Integer
    vData As Long
    PadBytes As Long
End Type

Public Type STATSTG
    atime As Currency
    cbSize As Currency
    clsid As GUID
    ctime As Currency
    grfLocksSupported As LOCKTYPE
    grfMode As STGM
    grfStateBits As Long
    mtime As Currency
    pwcsName As Long
    reserved As Long
    eType As STGTY
End Type

' ----==== Ole32 API-Deklarationen ====----
Private Declare Function IIDFromString Lib "ole32.dll" ( _
                         ByVal lpsz As Long, _
                         ByRef lpIID As IID) As Long

Private Declare Function CreateStreamOnHGlobal Lib "ole32.dll" ( _
                         ByVal hGlobal As Long, _
                         ByVal fDeleteOnRelease As Long, _
                         ByRef ppstm As Any) As Long

' ----==== Oleaut32 API-Deklarationen ====----
Private Declare Function OleLoadPicture Lib "oleaut32.dll" ( _
                         ByVal lpstream As Long, _
                         ByVal lSize As Long, _
                         ByVal fRunmode As Long, _
                         ByRef riid As IID, _
                         ByRef lplpvObj As Any) As Long

' ----==== Shell32 API-Deklarationen ====----
Private Declare Function SHCreateItemFromParsingName Lib "shell32.dll" ( _
                         ByVal pszPath As Long, _
                         ByVal pbc As Long, _
                         ByRef riid As IID, _
                         ByRef pUnk As Long) As Long

Private Declare Function SHGetPropertyStoreFromParsingName Lib "shell32.dll" ( _
                         ByVal pszPath As Long, _
                         ByVal pbc As Long, _
                         ByVal flags As Long, _
                         ByRef riid As IID, _
                         ByRef ppv As Long) As Long

' ----==== Shlwapi API-Deklarationen ====----
Private Declare Function SHCreateStreamOnFileEx Lib "shlwapi.dll" ( _
                         ByVal pszFile As Long, _
                         ByVal grfMode As STGM, _
                         ByVal dwAttributes As Long, _
                         ByVal fCreate As Long, _
                         ByVal pstmTemplate As Long, _
                         ByRef ppstm As Long) As Long

' ----==== Propsys API-Deklarationen ====----
Private Declare Function PSGetPropertyKeyFromName Lib "propsys.dll" ( _
                         ByVal pszName As Long, _
                         ByRef propkey As PROPERTYKEY) As Long

' ----==== Konvertiert ein Bild und speichert es auf die Festplatte ====----
Public Function TranscodeImageToDisc(ByVal ImageFile As String, Optional ByVal _
    ConvertTo As TI_FLAGS = TI_BITMAP) As Boolean

    Dim bRet As Boolean
    Dim lWidth As Long
    Dim lHeight As Long
    Dim tIID As IID
    Dim tSize As SIZE
    Dim sOutImageFile As String
    Dim pImageFile As Long
    Dim psOutImageFile As Long
    Dim pIStream As Long
    Dim pIShellItem As Long
    Dim pIID_IShellItem As Long
    Dim IStream As clsIStream
    Dim IShellItem As clsIShellItem
    Dim ITranscodeImage As clsITranscodeImage

    bRet = False

    ' existiert die Datei
    If FileExists(ImageFile) = True Then

        ' Dimensionen des Bildes vom IPropertyStore ermitteln
        ' Hier kann auch eine andere Gre angegeben werden. Das Bild wird
        ' dann entsprechend proportional skaliert.
        tSize = GetImageDimension(ImageFile)

        ' IID_IShellItem -> Type IID
        pIID_IShellItem = StrPtr(IID_IShellItem)
        If IIDFromString(pIID_IShellItem, tIID) = S_OK Then

            ' ein IShellItem-Interface von Datei erstellen
            pImageFile = StrPtr(ImageFile)
            If SHCreateItemFromParsingName(pImageFile, 0&, tIID, pIShellItem) = _
                S_OK Then

                ' Klasse fr IShellItem initialisieren
                Set IShellItem = New clsIShellItem
                Call IShellItem.Initialize(pIShellItem)

                ' zu Bitmap konvertieren
                If ConvertTo = TI_BITMAP Then

                    ' Dateiname fr die Ausgabe erstellen
                    sOutImageFile = Replace$(ImageFile, GetFileExtension( _
                        ImageFile), "bmp")

                Else ' zu JPEG konvertieren
                
                    ' Dateiname fr die Ausgabe erstellen
                    sOutImageFile = Replace$(ImageFile, GetFileExtension( _
                        ImageFile), "jpg")

                End If

                ' FileStream fr die Ausgabedatei erstellen -> IStream-Interface
                psOutImageFile = StrPtr(sOutImageFile)
                If SHCreateStreamOnFileEx(psOutImageFile, STGM_CREATE Or _
                    STGM_WRITE, 0&, 0&, 0&, pIStream) = S_OK Then

                    ' Klasse fr IStream initialisieren
                    Set IStream = New clsIStream
                    Call IStream.Initialize(pIStream)

                    ' ITranscodeImage-Interface erstellen
                    Set ITranscodeImage = New clsITranscodeImage

                    ' IShellItem konvertieren und in den FileStream schreiben
                    If ITranscodeImage.TranscodeImage(pIShellItem, tSize.cx, _
                        tSize.cy, ConvertTo, pIStream, lWidth, lHeight) = True _
                        Then

                        bRet = True

                    End If

                    ' Aufrumen
                    Set ITranscodeImage = Nothing

                    ' Aufrumen
                    Set IStream = Nothing

                End If

                ' Aufrumen
                Set IShellItem = Nothing

            End If

        End If

    End If

    TranscodeImageToDisc = bRet

End Function

' ----==== Konvertiert ein Bild und erstellt ein StdPicture ====----
Public Function TranscodeImageToPicture(ByVal ImageFile As String) As StdPicture

    Dim lWidth As Long
    Dim lHeight As Long
    Dim lStreamSize As Long
    Dim tIID As IID
    Dim tSize As SIZE
    Dim pImageFile As Long
    Dim pIStream As Long
    Dim pIShellItem As Long
    Dim pIID_IShellItem As Long
    Dim pIID_IPicture As Long
    Dim oPic As StdPicture
    Dim IPic As IPicture
    Dim IStream As clsIStream
    Dim IShellItem As clsIShellItem
    Dim ITranscodeImage As clsITranscodeImage

    ' existiert die Datei
    If FileExists(ImageFile) = True Then

        ' Dimensionen des Bildes vom IPropertyStore ermitteln
        ' Hier kann auch eine andere Gre angegeben werden. Das Bild wird
        ' dann entsprechend proportional skaliert.
        tSize = GetImageDimension(ImageFile)

        ' IID_IShellItem -> Type IID
        pIID_IShellItem = StrPtr(IID_IShellItem)
        If IIDFromString(pIID_IShellItem, tIID) = S_OK Then

            ' ein IShellItem-Interface von Datei erstellen
            pImageFile = StrPtr(ImageFile)
            If SHCreateItemFromParsingName(pImageFile, 0&, tIID, pIShellItem) = _
                S_OK Then

                ' Klasse fr IShellItem initialisieren
                Set IShellItem = New clsIShellItem
                Call IShellItem.Initialize(pIShellItem)

                ' einen IStream im Speicher erstellen
                If CreateStreamOnHGlobal(0&, True, pIStream) = S_OK Then

                    ' Klasse fr IStream initialisieren
                    Set IStream = New clsIStream
                    Call IStream.Initialize(pIStream)

                    ' ITranscodeImage-Interface erstellen
                    Set ITranscodeImage = New clsITranscodeImage

                    ' IShellItem konvertieren und in den IStream schreiben
                    If ITranscodeImage.TranscodeImage(pIShellItem, tSize.cx, _
                        tSize.cy, TI_BITMAP, pIStream, lWidth, lHeight) = True _
                        Then

                        ' IID_IPicture -> Type IID
                        pIID_IPicture = StrPtr(IID_IPicture)
                        If IIDFromString(pIID_IPicture, tIID) = S_OK Then

                            ' Gre des IStream ermitteln
                            lStreamSize = CLng(IStream.Seek_Stream(0, STREAM_SEEK_END))

                            ' zurck an den Anfang des IStreams
                            Call IStream.Seek_Stream(0, STREAM_SEEK_SET)

                            ' IPicture vom IStream erstellen
                            If OleLoadPicture(pIStream, lStreamSize, False, tIID, _
                                IPic) = S_OK Then

                                ' IPicture -> StdPicture
                                Set oPic = IPic

                                ' Aufrumen
                                Set IPic = Nothing

                            End If
                        End If
                    End If

                    ' Aufrumen
                    Set ITranscodeImage = Nothing

                    ' Aufrumen
                    Set IStream = Nothing

                End If

                ' Aufrumen
                Set IShellItem = Nothing

            End If
        End If
    End If

    Set TranscodeImageToPicture = oPic

End Function

' ----==== Ermitteln der Bilddimensionen ====----
Private Function GetImageDimension(ByVal ImageFile As String) As SIZE

    Dim psIID As Long
    Dim pImageFile As Long
    Dim pImageWidth As Long
    Dim pImageHeight As Long
    Dim pIPropertyStore As Long
    Dim tIID As IID
    Dim tSize As SIZE
    Dim tPropImgWidth As PROPERTYKEY
    Dim tPropImgHeight As PROPERTYKEY
    Dim tPropVarWidth As PROPVARIANT
    Dim tPropVarHeight As PROPVARIANT
    Dim IPropertyStore As clsIPropertyStore

    ' existiert die Datei
    If FileExists(ImageFile) = True Then

        ' Canonical-Name -> PROPERTYKEY
        pImageWidth = StrPtr(ImageWidth)
        If PSGetPropertyKeyFromName(pImageWidth, tPropImgWidth) = S_OK Then

            ' Canonical-Name -> PROPERTYKEY
            pImageHeight = StrPtr(ImageHeight)
            If PSGetPropertyKeyFromName(pImageHeight, tPropImgHeight) = S_OK Then

                ' IID_IPropertyStore -> Type IID
                psIID = StrPtr(IID_IPropertyStore)
                If IIDFromString(psIID, tIID) = S_OK Then

                    ' IPropertyStore-Interface von Datei erstellen
                    pImageFile = StrPtr(ImageFile)
                    If SHGetPropertyStoreFromParsingName(pImageFile, 0&, _
                        GPS_DEFAULT, tIID, pIPropertyStore) = S_OK Then

                        ' Klasse fr IPropertyStore initialisieren
                        Set IPropertyStore = New clsIPropertyStore
                        Call IPropertyStore.Initialize(pIPropertyStore)

                        ' PROPERTYKEY vom IPropertyStore auslesen -> PROPVARIANT
                        tPropVarWidth = IPropertyStore.GetValue(tPropImgWidth)
                        tPropVarHeight = IPropertyStore.GetValue(tPropImgHeight)

                        ' Fr System.Image.HorizontalSize und System.Image.VerticalSize
                        ' ist der Variant-Type = VT_UI4
                        If tPropVarWidth.vt = VT_UI4 Then
                            If tPropVarHeight.vt = VT_UI4 Then

                                ' Werte stehe dann in PROPVARIANT.vData -> Type Size
                                tSize.cx = tPropVarWidth.vData
                                tSize.cy = tPropVarHeight.vData

                            End If
                        End If

                        ' Aufrumen
                        Set IPropertyStore = Nothing

                    End If
                End If
            End If
        End If
    End If

    GetImageDimension = tSize

End Function

' ----==== Existiert die Datei ====----
Private Function FileExists(FileName As String) As Boolean

    On Error Resume Next

    Dim ret As Long

    ret = Len(Dir$(FileName))

    If Err Or ret = 0 Then FileExists = False Else FileExists = True

End Function

' ----==== Dateierweiterung ermitteln ====----
Private Function GetFileExtension(ByVal ImageFile As String) As String

    GetFileExtension = Mid$(ImageFile, InStrRev(ImageFile, ".") + 1, Len(ImageFile))

End Function

